@@ -1,9 +1,12 @@
 """Module containing the CLI programs for histoprint."""
 
-import numpy as np
+from io import StringIO
+
 import click
-from histoprint import *
+import numpy as np
+
 import histoprint.formatter as formatter
+from histoprint import *
 
 
 @click.command()
@@ -67,21 +70,25 @@ import histoprint.formatter as formatter
     "starting at 0. For CSV files, the fields must be the names of the columns "
     "as specified in the first line of the file. When plotting from ROOT files, "
     "at least one field must be specified. This can either be the path to a "
-    "single TH1, or one or more paths to TTree branches.",
+    "single TH1, or one or more paths to TTree branches. Also supports slicing "
+    "of array-like branches, e.g. use 'tree/branch[:,2]' to histogram the 3rd "
+    "elements of a vector-like branch.",
 )
 @click.option(
     "-c",
     "--columns",
     type=int,
     default=None,
-    help="Total width of the displayed historgram in characters. Defaults to width of the terminal.",
+    help="Total width of the displayed historgram in characters. Defaults to "
+    "width of the terminal.",
 )
 @click.option(
     "-r",
     "--lines",
     type=int,
     default=None,
-    help="Approximate total height of the displayed historgram in characters. Calculated from number of columns by default.",
+    help="Approximate total height of the displayed historgram in characters. "
+    "Calculated from number of columns by default.",
 )
 @click.version_option()
 def histoprint(infile, **kwargs):
@@ -99,12 +106,6 @@ def histoprint(infile, **kwargs):
         data = ""
 
     if len(data) > 0:
-        try:
-            # Python 2
-            from StringIO import StringIO
-        except ImportError:
-            # Python 3
-            from io import StringIO
         data_handle = StringIO(data)
         del data
 
@@ -129,7 +130,7 @@ def histoprint(infile, **kwargs):
         _histoprint_root(infile, **kwargs)
         exit(0)
     except ImportError:
-        click.echo("Cannot try ROOT file format. Uproot module not found.", err=True)
+        pass
 
     click.echo("Could not interpret file format.", err=True)
     exit(1)
@@ -223,7 +224,18 @@ def _histoprint_csv(infile, **kwargs):
 def _histoprint_root(infile, **kwargs):
     """Interpret file as as ROOT file."""
 
-    import uproot as up
+    # Import uproot
+    try:
+        import uproot as up
+    except ImportError:
+        click.echo("Cannot try ROOT file format. Uproot module not found.", err=True)
+        raise
+    # Import awkward
+    try:
+        import awkward as ak
+    except ImportError:
+        click.echo("Cannot try ROOT file format. Awkward module not found.", err=True)
+        raise
 
     # Open root file
     F = up.open(infile)
@@ -231,7 +243,7 @@ def _histoprint_root(infile, **kwargs):
     # Interpret field names
     fields = list(kwargs.pop("fields", []))
     if len(fields) == 0:
-        click.echo("Must specify at least on field for ROOT files.", err=True)
+        click.echo("Must specify at least one field for ROOT files.", err=True)
         click.echo(F.keys())
         exit(1)
 
@@ -241,20 +253,31 @@ def _histoprint_root(infile, **kwargs):
 
     # Read the data
     if len(fields) == 1:
-        # Possible a single histogram
+        # Possibly a single histogram
         try:
-            hist = F[fields[0]].numpy()
-        except (AttributeError, KeyError):
-            pass
+            hist = F[fields[0]]
+        except KeyError:
+            pass  # Deal with key error further down
         else:
-            kwargs.pop("bins", None)  # Get rid of useless parameter
-            print_hist(hist, **kwargs)
-            return
+            try:
+                hist = F[fields[0]].to_numpy()
+            except AttributeError:
+                pass
+            else:
+                kwargs.pop("bins", None)  # Get rid of useless parameter
+                print_hist(hist, **kwargs)
+                return
 
     data = []
     for field in fields:
         branch = F
+        index = ""
         for key in field.split("/"):
+            # Get possible array indices
+            if "[" in key and key[-1] == "]":
+                i = key.find("[")
+                index = key[i:]
+                key = key[:i]
             try:
                 branch = branch[key]
             except KeyError:
@@ -263,14 +286,32 @@ def _histoprint_root(infile, **kwargs):
                     % (key, branch.keys())
                 )
                 exit(1)
-        try:
-            d = np.array(branch.array().flatten())
-        except ValueError:
-            click.echo(
-                "Could not interpret root object '%s'. Possible child branches: %s"
-                % (key, branch.keys())
-            )
+        try:  # Does the object have values?
+            d = branch.array()
+        except (AttributeError, TypeError):
+            try:
+                click.echo(
+                    "Could not interpret root object '%s'. Possible child branches: %s"
+                    % (key, branch.keys())
+                )
+            except AttributeError:
+                click.echo("Could not interpret root object '%s'." % (key))
             exit(1)
+
+        # Apply index
+        try:
+            d = eval("d" + index)
+        except Exception:
+            raise
+
+        # Flatten if necessary
+        try:
+            d = ak.flatten(d)
+        except ValueError:
+            pass
+
+        # Turn into flat numpy array
+        d = ak.to_numpy(d)
         data.append(d)
 
     # Interpret bins
